@@ -2,9 +2,9 @@ try:
     get_ipython
 except NameError:
     import matplotlib
-
     matplotlib.use("Agg")
 
+
 import sys
 import logging
 import warnings
@@ -45,8 +45,8 @@ from .tools import (
     calc_kBET,
     calc_kSIM,
     diffmap,
-    reduce_diffmap_to_3d,
     calc_pseudotime,
+    jump_method,
     cluster,
     louvain,
     leiden,
