@@ -1,4 +1,10 @@
-from .utils import update_rep, X_from_rep, W_from_rep, slicing, calc_mean, calc_mean_and_var, calc_expm1, calc_stat_per_batch, normalize_by_count, calc_sig_background, simulate_doublets
+import psutil
+_cpu_count = psutil.cpu_count(logical=False)
+if _cpu_count is None:
+    _cpu_count = psutil.cpu_count(logical=True)
+
+
+from .utils import eff_n_jobs, update_rep, X_from_rep, W_from_rep, slicing, calc_mean, calc_mean_and_var, calc_expm1, calc_stat_per_batch, normalize_by_count, calc_sig_background, simulate_doublets
 
 from .preprocessing import (
     qc_metrics,
@@ -25,9 +31,9 @@ from .nearest_neighbors import (
     calc_kSIM,
 )
 from .graph_operations import construct_graph
-from .diffusion_map import diffmap, reduce_diffmap_to_3d
+from .diffusion_map import diffmap
 from .pseudotime import calc_pseudotime, infer_path
-from .clustering import louvain, leiden, spectral_louvain, spectral_leiden, cluster
+from .clustering import jump_method, louvain, leiden, spectral_louvain, spectral_leiden, cluster
 from .net_regressor import net_train_and_predict
 from .visualization import (
     tsne,
