@@ -4,10 +4,11 @@ import pandas as pd
 from pegasusio import MultimodalData
 from natsort import natsorted
 
+from threadpoolctl import threadpool_limits
 from sklearn.cluster import KMeans
-from typing import List, Optional
+from typing import List, Optional, Union
 
-from pegasus.tools import construct_graph
+from pegasus.tools import eff_n_jobs, construct_graph, calc_stat_per_batch
 
 import logging
 logger = logging.getLogger(__name__)
@@ -15,6 +16,122 @@ logger = logging.getLogger(__name__)
 from pegasusio import timer
 
 
+def _calc_trans_distor(X: np.ndarray, labels: np.ndarray, Y: float) -> float:
+    """ Calculate transformed distortion function for the jump method  """
+    _, means, _ = calc_stat_per_batch(X, labels)
+    distor = (((X - means.T[labels,:]) ** 2).sum() / X.shape[0] / X.shape[1])
+    trans_distor = distor ** (-Y)
+    return trans_distor
+
+@timer(logger=logger)
+def jump_method(
+    data: MultimodalData,
+    rep: str = "pca",
+    K_max: int = 40,
+    Y: float = None,
+    n_jobs: int = -1,
+    random_state: int = 0,
+) -> None:
+    """ Determine the optimal number of clusters using the Jump Method. [Sugar and James, 2003]_
+
+    Parameters
+    ----------
+    data: ``pegasusio.MultimodalData``
+        Annotated data matrix with rows for cells and columns for genes.
+
+    rep: ``str``, optional, default: ``"pca"``
+        The embedding representation used for clustering. Keyword ``'X_' + rep`` must exist in ``data.obsm``. By default, use PCA coordinates.
+    
+    K_max: ``int``, optional, default: 40
+        The maximum number of clusters to try.
+
+    Y: ``float``, optional, default: ``None``
+        The transformation power used. If None, use min(data.shape[1] / 3.0, 3.0). 
+
+    n_jobs : `int`, optional (default: -1)
+        Number of threads to use. -1 refers to using all physical CPU cores.
+
+    random_state: ``int``, optional, default: ``0``
+        Random seed for reproducing results.
+
+    Returns
+    -------
+    ``None``
+
+    Update ``data.uns``:
+        * ``data.obs[jump_values]``: Jump values (difference of adjacent transformed distortion values)
+
+    Examples
+    --------
+    >>> pg.jump_method(data)
+    """
+    X = data.obsm[f"X_{rep}"]
+    Y = min(data.shape[1] / 3.0, 3.0) if Y is None else Y
+    logger.info(f"Jump method: Y = {Y:.3f}.")
+
+    n_jobs = eff_n_jobs(n_jobs)
+    jump_values = np.zeros(K_max, dtype = np.float64)
+    v_old = v = 0.0
+    for k in range(1, K_max + 1):
+        with threadpool_limits(limits = n_jobs):
+            kmeans = KMeans(n_clusters = k, random_state = random_state).fit(X)
+        v = _calc_trans_distor(X, kmeans.labels_, Y)
+        jump_values[k - 1] = v - v_old
+        v_old = v
+        logger.info(f"K = {k} is finished, jump_value = {jump_values[k - 1]:.6f}.")
+    optimal_k = np.argmax(jump_values) + 1
+
+    data.uns[f"{rep}_jump_values"] = jump_values
+    data.uns[f"{rep}_optimal_k"] = optimal_k
+
+    logger.info(f"Jump method finished. Optimal K = {optimal_k}.")
+
+
+def _run_community_detection(algo, module, G, resolution, random_state, n_iter = -1):
+    partition_type = module.RBConfigurationVertexPartition
+    if algo == "louvain":
+        partition = partition_type(G, resolution_parameter=resolution, weights="weight")
+        optimiser = module.Optimiser()
+        optimiser.set_rng_seed(random_state)
+        diff = optimiser.optimise_partition(partition)
+    else:
+        partition = module.find_partition(
+            G,
+            partition_type,
+            seed=random_state,
+            weights="weight",
+            resolution_parameter=resolution,
+            n_iterations=n_iter,
+        )
+    return partition.membership
+
+def _find_optimal_resolution(algo, module, optimal_k, resol_max, G, random_state, n_iter = -1):
+    resol = None
+    membership = None
+
+    resol_l = 0.01
+    resol_r = resol_max
+    while resol_r - resol_l > 0.05:
+        resol_mid = (resol_l + resol_r) / 2.0
+        membership_mid = _run_community_detection(algo, module, G, resol_mid, random_state, n_iter)
+        k = max(membership_mid) + 1
+        logger.info(f"_find_optimal_resolution: resol = {resol_mid:.4f}, k = {k}, optimal_k = {optimal_k}.")
+        if k >= optimal_k:
+            resol_r = resol_mid
+            resol = resol_mid
+            membership = membership_mid
+        else:
+            resol_l = resol_mid
+    
+    if resol is None:
+        resol = resol_r
+        membership = _run_community_detection(algo, module, G, resol, random_state, n_iter)
+        k = max(membership_mid) + 1
+        logger.info(f"_find_optimal_resolution: resol = {resol:.4f}, k = {k}, optimal_k = {optimal_k}.")
+
+    return resol, membership
+
+
 @timer(logger=logger)
 def louvain(
     data: MultimodalData,
@@ -53,7 +170,6 @@ def louvain(
     --------
     >>> pg.louvain(data)
     """
-
     try:
         import louvain as louvain_module
     except ImportError:
@@ -65,13 +181,18 @@ def louvain(
     W = data.uns[rep_key]
 
     G = construct_graph(W)
-    partition_type = louvain_module.RBConfigurationVertexPartition
-    partition = partition_type(G, resolution_parameter=resolution, weights="weight")
-    optimiser = louvain_module.Optimiser()
-    optimiser.set_rng_seed(random_state)
-    diff = optimiser.optimise_partition(partition)
+    if resolution is not None:
+        membership = _run_community_detection("louvain", louvain_module, G, resolution, random_state)    
+    else:
+        optimal_k_key = f"{rep}_optimal_k"
+        if optimal_k_key not in data.uns:
+            jump_method(data, rep = rep, random_state = random_state)
+        optimal_k = data.uns[optimal_k_key]
+        resol_max = 2.0
+        resolution, membership = _find_optimal_resolution("louvain", louvain_module, optimal_k, resol_max, G, random_state)
 
-    labels = np.array([str(x + 1) for x in partition.membership])
+    data.uns["louvain_resolution"] = resolution
+    labels = np.array([str(x + 1) for x in membership])
     categories = natsorted(np.unique(labels))
     data.obs[class_label] = pd.Categorical(values=labels, categories=categories)
 
@@ -133,17 +254,18 @@ def leiden(
     W = data.uns[rep_key]
 
     G = construct_graph(W)
-    partition_type = leidenalg.RBConfigurationVertexPartition
-    partition = leidenalg.find_partition(
-        G,
-        partition_type,
-        seed=random_state,
-        weights="weight",
-        resolution_parameter=resolution,
-        n_iterations=n_iter,
-    )
+    if resolution is not None:
+        membership = _run_community_detection("leiden", leidenalg, G, resolution, random_state, n_iter)        
+    else:
+        optimal_k_key = f"{rep}_optimal_k"
+        if optimal_k_key not in data.uns:
+            jump_method(data, rep = rep, random_state = random_state)
+        optimal_k = data.uns[optimal_k_key]
+        resol_max = 2.0
+        resolution, membership = _find_optimal_resolution("leiden", leidenalg, optimal_k, resol_max, G, random_state, n_iter)
 
-    labels = np.array([str(x + 1) for x in partition.membership])
+    data.uns["leiden_resolution"] = resolution
+    labels = np.array([str(x + 1) for x in membership])
     categories = natsorted(np.unique(labels))
     data.obs[class_label] = pd.Categorical(values=labels, categories=categories)
 
@@ -156,6 +278,7 @@ def partition_cells_by_kmeans(
     n_clusters: int,
     n_clusters2: int,
     n_init: int,
+    n_jobs: int,
     random_state: int,
     min_avg_cells_per_final_cluster: Optional[int] = 10,
 ) -> List[int]:
@@ -164,29 +287,32 @@ def partition_cells_by_kmeans(
     if n_clusters == 1:
         return np.zeros(X.shape[0], dtype = np.int32)
 
+    n_jobs = eff_n_jobs(n_jobs)
+
     kmeans_params = {
         'n_clusters': n_clusters,
         'n_init': n_init,
         'random_state': random_state,
     }
-
     km = KMeans(**kmeans_params)
-    km.fit(X)
-    coarse = km.labels_.copy()
 
-    km.set_params(n_init=1)
-    labels = coarse.copy()
-    base_sum = 0
-    for i in range(n_clusters):
-        idx = coarse == i
-        nc = min(n_clusters2, max(idx.sum() // min_avg_cells_per_final_cluster, 1))
-        if nc == 1:
-            labels[idx] = base_sum
-        else:
-            km.set_params(n_clusters=nc)
-            km.fit(X[idx, :])
-            labels[idx] = base_sum + km.labels_
-        base_sum += nc
+    with threadpool_limits(limits = n_jobs):
+        km.fit(X)
+        coarse = km.labels_.copy()
+
+        km.set_params(n_init=1)
+        labels = coarse.copy()
+        base_sum = 0
+        for i in range(n_clusters):
+            idx = coarse == i
+            nc = min(n_clusters2, max(idx.sum() // min_avg_cells_per_final_cluster, 1))
+            if nc == 1:
+                labels[idx] = base_sum
+            else:
+                km.set_params(n_clusters=nc)
+                km.fit(X[idx, :])
+                labels[idx] = base_sum + km.labels_
+            base_sum += nc
 
     return labels
 
@@ -200,6 +326,7 @@ def spectral_louvain(
     n_clusters: int = 30,
     n_clusters2: int = 50,
     n_init: int = 10,
+    n_jobs: int = -1,
     random_state: int = 0,
     class_label: str = "spectral_louvain_labels",
 ) -> None:
@@ -228,6 +355,9 @@ def spectral_louvain(
     n_init: ``int``, optional, default: ``10``
         Number of kmeans tries for the first level clustering. Default is set to be the same as scikit-learn Kmeans function.
 
+    n_jobs : `int`, optional (default: -1)
+        Number of threads to use for the KMeans step. -1 refers to using all physical CPU cores.
+
     random_state: ``int``, optional, default: ``0``
         Random seed for reproducing results.
 
@@ -245,7 +375,6 @@ def spectral_louvain(
     --------
     >>> pg.spectral_louvain(data)
     """
-
     try:
         import louvain as louvain_module
     except ImportError:
@@ -262,7 +391,7 @@ def spectral_louvain(
         raise ValueError("Cannot find affinity matrix. Please run neighbors first!")
 
     labels = partition_cells_by_kmeans(
-        data.obsm[rep_kmeans], n_clusters, n_clusters2, n_init, random_state,
+        data.obsm[rep_kmeans], n_clusters, n_clusters2, n_init, n_jobs, random_state,
     )
 
     W = data.uns["W_" + rep]
@@ -296,6 +425,7 @@ def spectral_leiden(
     n_clusters: int = 30,
     n_clusters2: int = 50,
     n_init: int = 10,
+    n_jobs: int = -1,
     random_state: int = 0,
     class_label: str = "spectral_leiden_labels",
 ) -> None:
@@ -324,6 +454,9 @@ def spectral_leiden(
     n_init: ``int``, optional, default: ``10``
         Number of kmeans tries for the first level clustering. Default is set to be the same as scikit-learn Kmeans function.
 
+    n_jobs : `int`, optional (default: -1)
+        Number of threads to use for the KMeans step. -1 refers to using all physical CPU cores.
+
     random_state: ``int``, optional, default: ``0``
         Random seed for reproducing results.
 
@@ -358,7 +491,7 @@ def spectral_leiden(
         raise ValueError("Cannot find affinity matrix. Please run neighbors first!")
 
     labels = partition_cells_by_kmeans(
-        data.obsm[rep_kmeans], n_clusters, n_clusters2, n_init, random_state,
+        data.obsm[rep_kmeans], n_clusters, n_clusters2, n_init, n_jobs, random_state,
     )
 
     W = data.uns["W_" + rep]
@@ -388,6 +521,7 @@ def cluster(
     algo: str = "louvain",
     rep: str = "pca",
     resolution: int = 1.3,
+    n_jobs: int = -1,
     random_state: int = 0,
     class_label: str = None,
     n_iter: int = -1,
@@ -415,6 +549,9 @@ def cluster(
     resolution: ``int``, optional, default: ``1.3``
         Resolution factor. Higher resolution tends to find more clusters.
 
+    n_jobs : `int`, optional (default: -1)
+        Number of threads to use for the KMeans step in 'spectral_louvain' and 'spectral_leiden'. -1 refers to using all physical CPU cores.
+
     random_state: ``int``, optional, default: ``0``
         Random seed for reproducing results.
 
@@ -470,6 +607,7 @@ def cluster(
                 "n_clusters": n_clusters,
                 "n_clusters2": n_clusters2,
                 "n_init": n_init,
+                "n_jobs": n_jobs,
             }
         )
 
