@@ -3,12 +3,16 @@ import numpy as np
 import pandas as pd
 
 from typing import List, Optional, Union, Tuple
+from matplotlib.figure import Figure
 
 import logging
 logger = logging.getLogger(__name__)
 
+from threadpoolctl import threadpool_limits
+
 from pegasusio import UnimodalData, MultimodalData
 from pegasusio import timer
+from pegasus.tools import eff_n_jobs
 
 
 
@@ -230,12 +234,11 @@ def _run_scrublet(
     expected_doublet_rate: Optional[float] = None,
     sim_doublet_ratio: Optional[float] = 2.0,
     n_prin_comps: Optional[int] = 30,
-    robust: Optional[bool] = False,
     k: Optional[int] = None,
     n_jobs: Optional[int] = -1,
     random_state: Optional[int] = 0,
     plot_hist: Optional[bool] = True
-) -> Union[None, "Figure"]:
+) -> Union[None, Figure]:
     """Calculate doublet scores using Scrublet-like [Wolock18]_ strategy for the current data.X; determine a right threshold based on the KDE curve.
        This function should be called after highly_variable_gene selection.
 
@@ -256,14 +259,11 @@ def _run_scrublet(
     n_prin_comps: ``int``, optional, default: ``30``
         Number of principal components.
 
-    robust: ``bool``, optional, default: ``False``.
-        If true, use 'arpack' instead of 'randomized' for large matrices (i.e. max(X.shape) > 500 and n_components < 0.8 * min(X.shape))
-
     k: ``int``, optional, default: ``None``
         Number of observed cell neighbors. If None, k = round(0.5 * sqrt(number of observed cells)). Total neighbors k_adj = round(k * (1.0 + sim_doublet_ratio)).
 
-    n_job: ``int``, optional, default: ``-``
-        Number of threads to use. If ``-1``, use all available threads.
+    n_jobs: ``int``, optional, default: ``-``
+        Number of threads to use. If ``-1``, use all physical CPU cores.
 
     random_state: ``int``, optional, default: ``0``
         Random state for doublet simulation, PCA and approximate nearest neighbor search.
@@ -318,9 +318,11 @@ def _run_scrublet(
     obsX -= m1 # standardize
     obsX /= std
 
-    svd_solver = "auto" if not robust else ("arpack" if max(obsX.shape) > 500 and n_prin_comps < 0.8 * min(obsX.shape) else "full") # PCA
-    pca = PCA(n_components=n_prin_comps, random_state=random_state, svd_solver=svd_solver)
-    obs_pca = pca.fit_transform(obsX)
+    pca = PCA(n_components=n_prin_comps, random_state=random_state)
+    n_jobs = eff_n_jobs(n_jobs)
+    with threadpool_limits(limits = n_jobs):
+        obs_pca = pca.fit_transform(obsX.astype(np.float64)) # float64 for reproducibility
+        obs_pca = np.ascontiguousarray(obs_pca, dtype=np.float32)
 
     # standardize and calculate PCA for sim_rawX
     simX = sim_rawX.astype(np.float32).toarray()
@@ -330,6 +332,7 @@ def _run_scrublet(
     simX /= std
 
     sim_pca = pca.transform(simX) # transform to PC coordinates
+    sim_pca = np.ascontiguousarray(sim_pca, dtype=np.float32)
 
     # concatenate observed and simulated data
     pc_coords = np.vstack((obs_pca, sim_pca))
@@ -421,6 +424,13 @@ def _run_scrublet(
 def _identify_doublets_fisher(cluster_labels: Union[pd.Categorical, List[int]], pred_dbl: List[bool], alpha: float = 0.05) -> pd.DataFrame:
     df = pd.crosstab(cluster_labels, pred_dbl)
 
+    if df.shape[1] == 1: # either no doublets or all doublets
+        result = pd.DataFrame({'cluster': df.index})
+        result['percentage'] = 100.0 if (True in df.columns) else 0.0
+        result['pval'] = 1.0
+        result['qval'] = 1.0
+        return result
+
     ndbl = df[True].sum()
     a = df[True].values.astype(np.int32)
     b = df[False].values.astype(np.int32)
@@ -454,16 +464,15 @@ def infer_doublets(
     expected_doublet_rate: Optional[float] = None,
     sim_doublet_ratio: Optional[float] = 2.0,
     n_prin_comps: Optional[int] = 30,
-    robust: Optional[bool] = False,
     k: Optional[int] = None,
     n_jobs: Optional[int] = -1,
     alpha: Optional[float] = 0.05,
     random_state: Optional[int] = 0,
     plot_hist: Optional[str] = "sample",
 ) -> None:
-    """Infer doublets using a Scrublet-like strategy. [Li20-2]_
+    """Infer doublets by first calculating Scrublet-like [Wolock18]_ doublet scores and then smartly determining an appropriate doublet score cutoff [Li20-2]_ .
 
-    This function must be called after clustering. 
+    This function should be called after clustering if clust_attr is not None. In this case, we will test if each cluster is significantly enriched for doublets using Fisher's exact test.
 
     Parameters
     ----------
@@ -488,14 +497,11 @@ def infer_doublets(
     n_prin_comps: ``int``, optional, default: ``30``
         Number of principal components.
 
-    robust: ``bool``, optional, default: ``False``.
-        If true, use 'arpack' instead of 'randomized' for large matrices (i.e. max(X.shape) > 500 and n_components < 0.8 * min(X.shape))
-
     k: ``int``, optional, default: ``None``
         Number of observed cell neighbors. If None, k = round(0.5 * sqrt(number of observed cells)). Total neighbors k_adj = round(k * (1.0 + sim_doublet_ratio)).
 
     n_jobs: ``int``, optional, default: ``-1``
-        Number of threads to use. If ``-1``, use all available threads.
+        Number of threads to use. If ``-1``, use all physical CPU cores.
 
     alpha: ``float``, optional, default: ``0.05``
         FDR significant level for cluster-level fisher exact test.
@@ -530,8 +536,7 @@ def infer_doublets(
     if channel_attr is None:
         if data.shape[0] >= min_cell:
             fig = _run_scrublet(data, expected_doublet_rate = expected_doublet_rate, sim_doublet_ratio = sim_doublet_ratio, \
-                                n_prin_comps = n_prin_comps, robust = robust, k = k, n_jobs = n_jobs, random_state = random_state, \
-                                plot_hist = if_plot)
+                                n_prin_comps = n_prin_comps, k = k, n_jobs = n_jobs, random_state = random_state, plot_hist = if_plot)
             if if_plot:
                 fig.savefig(f"{plot_hist}.dbl.png")
         else:
@@ -564,8 +569,7 @@ def infer_doublets(
                 highly_variable_features(unidata)
                 # Run _run_scrublet
                 fig = _run_scrublet(unidata, name = channel, expected_doublet_rate = expected_doublet_rate, sim_doublet_ratio = sim_doublet_ratio, \
-                                    n_prin_comps = n_prin_comps, robust = robust, k = k, n_jobs = n_jobs, random_state = random_state, \
-                                    plot_hist = if_plot)
+                                    n_prin_comps = n_prin_comps, k = k, n_jobs = n_jobs, random_state = random_state, plot_hist = if_plot)
                 if if_plot:
                     fig.savefig(f"{plot_hist}.{channel}.dbl.png")
 
