@@ -3,12 +3,12 @@ import numpy as np
 import pandas as pd
 
 from scipy.sparse import issparse, csr_matrix
-
+from threadpoolctl import threadpool_limits
 from sklearn.decomposition import PCA, TruncatedSVD
 
 from typing import List, Tuple, Union
 from pegasusio import UnimodalData, MultimodalData, calc_qc_filters, DictWithDefault
-from pegasus.tools import normalize_by_count
+from pegasus.tools import eff_n_jobs, normalize_by_count
 
 import logging
 logger = logging.getLogger(__name__)
@@ -314,7 +314,7 @@ def log_norm(
         Use current selected modality in data, which should contain one RNA expression matrix.
 
     norm_count: ``int``, optional, default: ``1e5``.
-        Total count of cells after normalization.
+        Total counts of one cell after normalization.
 
     backup_matrix: ``str``, optional, default: ``raw.X``.
         The key name of the backup count matrix, usually the raw counts.
@@ -354,7 +354,6 @@ def select_features(
     features: str = "highly_variable_features",
     standardize: bool = True,
     max_value: float = 10.0,
-    use_cache: bool = True,
 ) -> str:
     """ Subset the features and store the resulting matrix in dense format in data.uns with `'_tmp_fmat_'` prefix, with the option of standardization and truncating based on max_value. `'_tmp_fmat_*'` will be removed before writing out the disk.
 
@@ -372,9 +371,6 @@ def select_features(
     max_value: ``float``, optional, default: ``10``.
         The threshold to truncate data after scaling. If ``None``, do not truncate.
 
-    use_cache: ``bool``, optional, default: ``False``.
-        Whether to use precalculated subset matrix if exists.
-
     Returns
     -------
     keyword: ``str``
@@ -389,33 +385,38 @@ def select_features(
     >>> pg.select_features(data)
     """
     keyword = "_tmp_fmat_" + str(features)  # fmat: feature matrix
+    batch_ls_key = "_tmp_ls_" + str(features)
 
-    if (not use_cache) or (use_cache and (keyword not in data.uns)):
+    if batch_ls_key in data.uns:
+        X = data.uns[keyword]
+    else:
         if features is not None:
             assert features in data.var
             X = data.X[:, data.var[features].values]
         else:
             X = data.X
-
+        
         from pegasus.tools import slicing
-        data.uns[keyword] = slicing(X, copy=True)
+        X = slicing(X, copy=True)
+        
+    if standardize:
+        m1 = X.mean(axis=0)
+        psum = np.multiply(X, X).sum(axis=0)
+        std = ((psum - X.shape[0] * (m1 ** 2)) / (X.shape[0] - 1.0)) ** 0.5
+        std[std == 0] = 1
+        X -= m1
+        X /= std
+        data.uns["stdzn_mean"] = m1
+        data.uns["stdzn_std"] = std
+        data.uns.pop(batch_ls_key, None)
 
-    if standardize or (max_value is not None):
-        X = data.uns[keyword]
-        if standardize:
-            m1 = X.mean(axis=0)
-            psum = np.multiply(X, X).sum(axis=0)
-            std = ((psum - X.shape[0] * (m1 ** 2)) / (X.shape[0] - 1.0)) ** 0.5
-            std[std == 0] = 1
-            X -= m1
-            X /= std
-            data.uns["stdzn_mean"] = m1
-            data.uns["stdzn_std"] = std
-        if max_value is not None:
-            data.uns["stdzn_max_value"] = max_value
-            X[X > max_value] = max_value
-            X[X < -max_value] = -max_value
-        data.uns[keyword] = X
+    if max_value is not None:
+        data.uns["stdzn_max_value"] = max_value
+        X[X > max_value] = max_value
+        X[X < -max_value] = -max_value
+        data.uns.pop(batch_ls_key, None)
+
+    data.uns[keyword] = X
 
     return keyword
 
@@ -427,9 +428,8 @@ def pca(
     features: str = "highly_variable_features",
     standardize: bool = True,
     max_value: float = 10.0,
-    robust: bool = False,
+    n_jobs: int = -1,
     random_state: int = 0,
-    use_cache: bool = True,
 ) -> None:
     """Perform Principle Component Analysis (PCA) to the data.
 
@@ -452,15 +452,12 @@ def pca(
     max_value: ``float``, optional, default: ``10``.
         The threshold to truncate data after scaling. If ``None``, do not truncate.
 
-    robust: ``bool``, optional, default: ``False``.
-        If true, use 'arpack' instead of 'randomized' for large matrices (i.e. max(X.shape) > 500 and n_components < 0.8 * min(X.shape))
+    n_jobs : `int`, optional (default: -1)
+        Number of threads to use. -1 refers to using all physical CPU cores.
 
     random_state: ``int``, optional, default: ``0``.
         Random seed to be set for reproducing result.
 
-    use_cache: ``bool``, optional, default: ``True``.
-        Whether to use precalculated subset feature matrix if exists.
-
     Returns
     -------
     ``None``.
@@ -481,19 +478,16 @@ def pca(
     --------
     >>> pg.pca(data)
     """
-    keyword = select_features(data, features=features, standardize=standardize, max_value=max_value, use_cache=use_cache)
-    X = data.uns[keyword]
+    keyword = select_features(data, features=features, standardize=standardize, max_value=max_value)
+    X = data.uns[keyword].astype(np.float64) # float64 to avoid precision issues and make results more reproducible across platforms
+    pca = PCA(n_components=n_components, random_state=random_state) # use auto solver, default is randomized for large datasets
 
-    if robust:
-        svd_solver = "arpack" if max(X.shape) > 500 and n_components < 0.8 * min(X.shape) else "full"
-        pca = PCA(n_components=n_components, random_state=random_state, svd_solver=svd_solver)
-    else:
-        pca = PCA(n_components=n_components, random_state=random_state)
+    n_jobs = eff_n_jobs(n_jobs)
+    with threadpool_limits(limits = n_jobs):
+        X_pca = pca.fit_transform(X)
 
-    X_pca = pca.fit_transform(X)
-
-    data.obsm["X_pca"] = np.ascontiguousarray(X_pca)
-    data.uns["PCs"] = np.ascontiguousarray(pca.components_.T)  # cannot be varm because numbers of features are not the same
+    data.obsm["X_pca"] = np.ascontiguousarray(X_pca, dtype=np.float32)
+    data.uns["PCs"] = np.ascontiguousarray(pca.components_.T, dtype=np.float32)  # cannot be varm because numbers of features are not the same
     data.uns["pca"] = {}
     data.uns["pca"]["variance"] = pca.explained_variance_
     data.uns["pca"]["variance_ratio"] = pca.explained_variance_ratio_
@@ -604,7 +598,7 @@ def regress_out(
         response_list.append(resid)
 
     pca_key = f'{rep}_regressed'
-    data.obsm[f'X_{pca_key}'] = np.vstack(response_list).T.astype(data.obsm[f'X_{rep}'].dtype)
+    data.obsm[f'X_{pca_key}'] = np.ascontiguousarray(np.vstack(response_list).T.astype(data.obsm[f'X_{rep}'].dtype), dtype=np.float32)
 
     return pca_key
 
@@ -614,7 +608,7 @@ def tsvd(
     data: MultimodalData,
     n_components: int = 51,
     features: str = "robust",
-    robust: bool = False,
+    n_jobs: int = -1,
     random_state: int = 0,
 ) -> None:
     """Perform Truncated Singular Value Decomposition (TSVD) to the data.
@@ -635,6 +629,9 @@ def tsvd(
     robust: ``bool``, optional, default: ``False``.
         If true, use 'arpack' instead of 'randomized'.
 
+    n_jobs : `int`, optional (default: -1)
+        Number of threads to use. -1 refers to using all physical CPU cores.
+
     random_state: ``int``, optional, default: ``0``.
         Random seed to be set for reproducing result.
 
@@ -657,12 +654,16 @@ def tsvd(
     X = data.X
     if (features is not None) and (data.var[features].sum() < data.shape[1]):
         X = X[:, data.var[features].values]
+    X = X.astype(np.float64) # for reproducible purpose
 
-    tsvd = TruncatedSVD(n_components = n_components, algorithm = 'arpack' if robust else 'randomized', random_state = random_state)
-    X_tsvd = tsvd.fit_transform(X)
+    tsvd = TruncatedSVD(n_components = n_components, random_state = random_state)
 
-    data.obsm["X_tsvd"] = np.ascontiguousarray(X_tsvd)
-    data.uns["TSVDs"] = np.ascontiguousarray(tsvd.components_.T)  # cannot be varm because numbers of features are not the same
+    n_jobs = eff_n_jobs(n_jobs)
+    with threadpool_limits(limits = n_jobs):
+        X_tsvd = tsvd.fit_transform(X)
+
+    data.obsm["X_tsvd"] = np.ascontiguousarray(X_tsvd, dtype=np.float32)
+    data.uns["TSVDs"] = np.ascontiguousarray(tsvd.components_.T, dtype=np.float32)  # cannot be varm because numbers of features are not the same
     data.uns["tsvd"] = {}
     data.uns["tsvd"]["variance"] = tsvd.explained_variance_
     data.uns["tsvd"]["variance_ratio"] = tsvd.explained_variance_ratio_
