@@ -2,6 +2,13 @@ import numpy as np
 import pandas as pd
 from scipy.sparse import issparse, csr_matrix
 from typing import Union, List, Tuple
+from anndata import AnnData
+
+from pegasus.tools import _cpu_count
+
+def eff_n_jobs(n_jobs: int) -> int:
+    """ If n_jobs < 0, set it as the number of physical cores _cpu_count """
+    return n_jobs if n_jobs > 0 else _cpu_count
 
 
 def update_rep(rep: str) -> str:
@@ -10,7 +17,7 @@ def update_rep(rep: str) -> str:
     return rep if rep is not None else "mat"
 
 
-def X_from_rep(data: "AnnData", rep: str) -> np.array:
+def X_from_rep(data: AnnData, rep: str) -> np.array:
     """
     If rep is not mat, first check if X_rep is in data.obsm. If not, raise an error.
     If rep is None, return data.X as a numpy array
@@ -24,7 +31,7 @@ def X_from_rep(data: "AnnData", rep: str) -> np.array:
         return data.X if not issparse(data.X) else data.X.toarray()
 
 
-def W_from_rep(data: "AnnData", rep: str) -> "csr_matrix":
+def W_from_rep(data: AnnData, rep: str) -> csr_matrix:
     """
     Return affinity matrix W based on representation rep.
     """
@@ -73,9 +80,14 @@ def calc_expm1(X: Union[csr_matrix, np.ndarray]) -> np.ndarray:
     return res
 
 
-def calc_stat_per_batch(X: Union[csr_matrix, np.ndarray], batch: pd.Categorical) -> Tuple[np.ndarray, np.ndarray, np.ndarray]:
-    nbatch = batch.categories.size
-    codes = batch.codes.astype(np.int32)
+def calc_stat_per_batch(X: Union[csr_matrix, np.ndarray], batch: Union[pd.Categorical, np.ndarray, list]) -> Tuple[np.ndarray, np.ndarray, np.ndarray]:
+    from pandas.api.types import is_categorical_dtype
+    if is_categorical_dtype(batch):
+        nbatch = batch.categories.size
+        codes = batch.codes.astype(np.int32)
+    else:
+        codes = np.array(batch, dtype = np.int32)
+        nbatch = codes.max() + 1 # assume cluster label starts from 0
 
     if issparse(X):
         from pegasus.cylib.fast_utils import calc_stat_per_batch_sparse
@@ -104,6 +116,9 @@ def calc_sig_background(X: Union[csr_matrix, np.ndarray], bins: pd.Categorical,
     n_bins = bins.categories.size
     codes = bins.codes.astype(np.int32)
 
+    if mean_vec.dtype == np.float32:
+        mean_vec = mean_vec.astype(np.float64)
+
     if issparse(X):
         from pegasus.cylib.fast_utils import calc_sig_background_sparse
         return calc_sig_background_sparse(X.shape[0], X.shape[1], X.data, X.indices, X.indptr, n_bins, codes, mean_vec)
