@@ -5,10 +5,11 @@ import umap as umap_module
 import forceatlas2 as fa2
 import uuid
 
+from threadpoolctl import threadpool_limits
 from pegasusio import MultimodalData
-from joblib import effective_n_jobs
 
 from pegasus.tools import (
+    eff_n_jobs,
     update_rep,
     X_from_rep,
     W_from_rep,
@@ -25,6 +26,8 @@ logger = logging.getLogger(__name__)
 
 from pegasusio import timer
 
+
+
 def calc_tsne(
     X,
     nthreads,
@@ -235,7 +238,7 @@ def tsne(
         Representation of data used for the calculation. By default, use PCA coordinates. If ``None``, use the count matrix ``data.X``.
 
     n_jobs: ``int``, optional, default: ``-1``
-        Number of threads to use. If ``-1``, use all available threads.
+        Number of threads to use. If ``-1``, use all physical CPU cores.
 
     n_components: ``int``, optional, default: ``2``
         Dimension of calculated FI-tSNE coordinates. By default, generate 2-dimensional data for 2D visualization.
@@ -271,7 +274,7 @@ def tsne(
     """
 
     rep = update_rep(rep)
-    n_jobs = effective_n_jobs(n_jobs)
+    n_jobs = eff_n_jobs(n_jobs)
     X = X_from_rep(data, rep).astype(np.float64)
 
     if learning_rate == "auto":
@@ -283,10 +286,10 @@ def tsne(
         if rep == "pca":
             initialization = X[:, 0:n_components].copy()
         else:
-            from sklearn.decomposition import PCA     
-            svd_solver = "arpack" if min(X.shape) > n_components else "full"
-            pca = PCA(n_components=n_components, random_state=random_state, svd_solver=svd_solver)
-            initialization = np.ascontiguousarray(pca.fit_transform(X))
+            from sklearn.decomposition import PCA
+            pca = PCA(n_components=n_components, random_state=random_state)
+            with threadpool_limits(limits = n_jobs):
+                initialization = np.ascontiguousarray(pca.fit_transform(X))
         initialization = initialization / np.std(initialization[:, 0]) * 0.0001
     else:
         assert isinstance(initialization, np.ndarray) and initialization.ndim == 2 and initialization.shape[0] == X.shape[0] and initialization.shape[1] == n_components
@@ -345,7 +348,7 @@ def umap(
         The effective scale of embedded data points.
 
     n_jobs: ``int``, optional, default: ``-1``
-        Number of threads to use for computing kNN graphs. If ``-1``, use all available threads.
+        Number of threads to use for computing kNN graphs. If ``-1``, use all physical CPU cores.
 
     full_speed: ``bool``, optional, default: ``False``
         * If ``True``, use multiple threads in constructing ``hnsw`` index. However, the kNN results are not reproducible.
@@ -424,7 +427,7 @@ def fle(
         Temporary file to store the coordinates as the input to forceatlas2. If ``None``, use ``tempfile.mkstemp`` to generate file name.
 
     n_jobs: ``int``, optional, default: ``-1``
-        Number of threads to use. If ``-1``, use all available threads.
+        Number of threads to use. If ``-1``, use all physical CPU cores.
 
     rep: ``str``, optional, default: ``"diffmap"``
         Representation of data used for the calculation. By default, use Diffusion Map coordinates. If ``None``, use the count matrix ``data.X``.
@@ -472,7 +475,7 @@ def fle(
         _, file_name = tempfile.mkstemp()
 
     rep = update_rep(rep)
-    n_jobs = effective_n_jobs(n_jobs)
+    n_jobs = eff_n_jobs(n_jobs)
 
     if ("W_" + rep) not in data.uns:
         neighbors(
@@ -562,6 +565,9 @@ def net_umap(
     rep: ``str``, optional, default: ``"pca"``
         Representation of data used for the calculation. By default, use PCA coordinates. If ``None``, use the count matrix ``data.X``.
 
+    n_jobs: ``int``, optional, default: ``-1``
+        Number of threads to use. If ``-1``, use all physical CPU cores.
+
     n_components: ``int``, optional, default: ``2``
         Dimension of calculated UMAP coordinates. By default, generate 2-dimensional data for 2D visualization.
 
@@ -618,7 +624,7 @@ def net_umap(
     """
 
     rep = update_rep(rep)
-    n_jobs = effective_n_jobs(n_jobs)
+    n_jobs = eff_n_jobs(n_jobs)
     knn_indices, knn_dists = get_neighbors(data, K = select_K, rep = rep, n_jobs = n_jobs, random_state = random_state, full_speed = full_speed)
 
     selected = select_cells(
@@ -671,7 +677,7 @@ def net_umap(
     Y_init = np.zeros((data.shape[0], n_components), dtype=np.float64)
     Y_init[selected, :] = X_umap
     Y_init[~selected, :] = net_train_and_predict(
-        X, X_umap, X_full[~selected, :], net_alpha, random_state, verbose=True
+        X, X_umap, X_full[~selected, :], net_alpha, n_jobs, random_state, verbose=True
     )
 
     data.obsm["X_" + out_basis + "_pred"] = Y_init
@@ -734,7 +740,7 @@ def net_fle(
         Temporary file to store the coordinates as the input to forceatlas2. If ``None``, use ``tempfile.mkstemp`` to generate file name.
 
     n_jobs: ``int``, optional, default: ``-1``
-        Number of threads to use. If ``-1``, use all available threads.
+        Number of threads to use. If ``-1``, use all physical CPU cores.
 
     rep: ``str``, optional, default: ``"diffmap"``
         Representation of data used for the calculation. By default, use Diffusion Map coordinates. If ``None``, use the count matrix ``data.X``.
@@ -801,7 +807,7 @@ def net_fle(
             _, file_name = tempfile.mkstemp()
 
     rep = update_rep(rep)
-    n_jobs = effective_n_jobs(n_jobs)
+    n_jobs = eff_n_jobs(n_jobs)
 
     if ("W_" + rep) not in data.uns:
         neighbors(
@@ -854,7 +860,7 @@ def net_fle(
     Y_init = np.zeros((data.shape[0], n_components), dtype=np.float64)
     Y_init[selected, :] = X_fle
     Y_init[~selected, :] = net_train_and_predict(
-        X, X_fle, X_full[~selected, :], net_alpha, random_state, verbose=True
+        X, X_fle, X_full[~selected, :], net_alpha, n_jobs, random_state, verbose=True
     )
 
     data.obsm["X_" + out_basis + "_pred"] = Y_init
